# -*- coding: utf-8 -*-


#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_tabTools(object):
    def setupUi(self, tabTools):
        tabTools.setObjectName("tabTools")
        tabTools.resize(589, 600)
        self.gridLayout_2 = QtWidgets.QGridLayout(tabTools)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setContentsMargins(6, 6, 6, 6)
        self.gridLayout.setSpacing(12)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.txtSearch = QtWidgets.QLineEdit(tabTools)
        self.txtSearch.setMinimumSize(QtCore.QSize(0, 24))
        self.txtSearch.setObjectName("txtSearch")
        self.horizontalLayout.addWidget(self.txtSearch)
        self.btnClear = QtWidgets.QToolButton(tabTools)
        self.btnClear.setMinimumSize(QtCore.QSize(0, 24))
        self.btnClear.setAutoRaise(True)
        self.btnClear.setObjectName("btnClear")
        self.horizontalLayout.addWidget(self.btnClear)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.labelHelp = QtWidgets.QLabel(tabTools)
        self.labelHelp.setText("")
        self.labelHelp.setWordWrap(True)
        self.labelHelp.setObjectName("labelHelp")
        self.gridLayout.addWidget(self.labelHelp, 2, 0, 1, 1)
        self.treeTools = QtWidgets.QTreeWidget(tabTools)
        self.treeTools.setAlternatingRowColors(True)
        self.treeTools.setIndentation(0)
        self.treeTools.setRootIsDecorated(False)
        self.treeTools.setAnimated(True)
        self.treeTools.setAllColumnsShowFocus(True)
        self.treeTools.setObjectName("treeTools")
        self.treeTools.headerItem().setText(0, "1")
        self.treeTools.header().setVisible(False)
        self.gridLayout.addWidget(self.treeTools, 1, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)

        self.retranslateUi(tabTools)
        self.btnClear.clicked.connect(self.txtSearch.clear)
        QtCore.QMetaObject.connectSlotsByName(tabTools)
        tabTools.setTabOrder(self.txtSearch, self.btnClear)
        tabTools.setTabOrder(self.btnClear, self.treeTools)

    def retranslateUi(self, tabTools):
        _translate = QtCore.QCoreApplication.translate
        tabTools.setWindowTitle(_translate("tabTools", "Tools"))
        self.txtSearch.setPlaceholderText(_translate("tabTools", "Search"))
        self.btnClear.setText(_translate("tabTools", "x"))
        self.treeTools.setSortingEnabled(False)
