# -*- coding: utf-8 -*-


#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Console(object):
    def setupUi(self, Console):
        Console.setObjectName("Console")
        Console.resize(755, 461)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Console)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.txtConsole = QtWidgets.QTextEdit(Console)
        self.txtConsole.setUndoRedoEnabled(False)
        self.txtConsole.setReadOnly(True)
        self.txtConsole.setTabStopWidth(40)
        self.txtConsole.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.txtConsole.setObjectName("txtConsole")
        self.horizontalLayout.addWidget(self.txtConsole)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setContentsMargins(6, 6, 6, 6)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.btnCopy = QtWidgets.QToolButton(Console)
        self.btnCopy.setEnabled(True)
        self.btnCopy.setIconSize(QtCore.QSize(24, 24))
        self.btnCopy.setAutoRaise(True)
        self.btnCopy.setObjectName("btnCopy")
        self.verticalLayout.addWidget(self.btnCopy)
        self.btnWordWrap = QtWidgets.QToolButton(Console)
        self.btnWordWrap.setIconSize(QtCore.QSize(24, 24))
        self.btnWordWrap.setCheckable(True)
        self.btnWordWrap.setAutoRaise(True)
        self.btnWordWrap.setObjectName("btnWordWrap")
        self.verticalLayout.addWidget(self.btnWordWrap)
        self.btnScrollDown = QtWidgets.QToolButton(Console)
        self.btnScrollDown.setIconSize(QtCore.QSize(24, 24))
        self.btnScrollDown.setCheckable(True)
        self.btnScrollDown.setChecked(True)
        self.btnScrollDown.setAutoRaise(True)
        self.btnScrollDown.setObjectName("btnScrollDown")
        self.verticalLayout.addWidget(self.btnScrollDown)
        self.btnTrash = QtWidgets.QToolButton(Console)
        self.btnTrash.setIconSize(QtCore.QSize(24, 24))
        self.btnTrash.setAutoRaise(True)
        self.btnTrash.setObjectName("btnTrash")
        self.verticalLayout.addWidget(self.btnTrash)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(Console)
        QtCore.QMetaObject.connectSlotsByName(Console)

    def retranslateUi(self, Console):
        _translate = QtCore.QCoreApplication.translate
        Console.setWindowTitle(_translate("Console", "Console"))
        self.btnCopy.setText(_translate("Console", "..."))
        self.btnWordWrap.setText(_translate("Console", "..."))
        self.btnScrollDown.setText(_translate("Console", "..."))
        self.btnTrash.setText(_translate("Console", "..."))
