# -*- coding: utf-8 -*-


#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_tabAbout(object):
    def setupUi(self, tabAbout):
        tabAbout.setObjectName("tabAbout")
        tabAbout.resize(346, 415)
        self.gridLayout = QtWidgets.QGridLayout(tabAbout)
        self.gridLayout.setContentsMargins(16, 16, 16, 16)
        self.gridLayout.setHorizontalSpacing(16)
        self.gridLayout.setVerticalSpacing(8)
        self.gridLayout.setObjectName("gridLayout")
        self.lblIcon = QtWidgets.QLabel(tabAbout)
        self.lblIcon.setText("")
        self.lblIcon.setObjectName("lblIcon")
        self.gridLayout.addWidget(self.lblIcon, 0, 0, 1, 1)
        self.lblTitle = QtWidgets.QLabel(tabAbout)
        font = QtGui.QFont()
        font.setPointSize(18)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitle.setFont(font)
        self.lblTitle.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitle.setObjectName("lblTitle")
        self.gridLayout.addWidget(self.lblTitle, 0, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(159, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 1)
        self.txtAbout = QtWidgets.QTextBrowser(tabAbout)
        self.txtAbout.setReadOnly(True)
        self.txtAbout.setOpenExternalLinks(True)
        self.txtAbout.setObjectName("txtAbout")
        self.gridLayout.addWidget(self.txtAbout, 1, 0, 1, 3)

        self.retranslateUi(tabAbout)
        QtCore.QMetaObject.connectSlotsByName(tabAbout)

    def retranslateUi(self, tabAbout):
        _translate = QtCore.QCoreApplication.translate
        tabAbout.setWindowTitle(_translate("tabAbout", "About"))
        self.lblTitle.setText(_translate("tabAbout", "Xappt QT"))
