# -*- coding: utf-8 -*-


#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_tabOptions(object):
    def setupUi(self, tabOptions):
        tabOptions.setObjectName("tabOptions")
        tabOptions.resize(263, 300)
        self.verticalLayout = QtWidgets.QVBoxLayout(tabOptions)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setContentsMargins(6, 6, 6, 6)
        self.gridLayout_3.setSpacing(12)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.cmbToolViewType = QtWidgets.QComboBox(tabOptions)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbToolViewType.sizePolicy().hasHeightForWidth())
        self.cmbToolViewType.setSizePolicy(sizePolicy)
        self.cmbToolViewType.setObjectName("cmbToolViewType")
        self.gridLayout_3.addWidget(self.cmbToolViewType, 2, 1, 1, 1)
        self.chkMinimizeToTray = QtWidgets.QCheckBox(tabOptions)
        self.chkMinimizeToTray.setObjectName("chkMinimizeToTray")
        self.gridLayout_3.addWidget(self.chkMinimizeToTray, 0, 1, 1, 1)
        self.chkStartMinimized = QtWidgets.QCheckBox(tabOptions)
        self.chkStartMinimized.setObjectName("chkStartMinimized")
        self.gridLayout_3.addWidget(self.chkStartMinimized, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(tabOptions)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 2, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_3)
        spacerItem = QtWidgets.QSpacerItem(20, 237, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(tabOptions)
        QtCore.QMetaObject.connectSlotsByName(tabOptions)
        tabOptions.setTabOrder(self.chkMinimizeToTray, self.chkStartMinimized)
        tabOptions.setTabOrder(self.chkStartMinimized, self.cmbToolViewType)

    def retranslateUi(self, tabOptions):
        _translate = QtCore.QCoreApplication.translate
        tabOptions.setWindowTitle(_translate("tabOptions", "Options"))
        self.chkMinimizeToTray.setText(_translate("tabOptions", "Minimize to system tray"))
        self.chkStartMinimized.setText(_translate("tabOptions", "Start Minimized"))
        self.label.setText(_translate("tabOptions", "View"))
